package com.uncc.tv;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.Properties;

import com.lhy.tool.util.Utils;

public class ParamManager {
	public static final String FORMAT_PARAMETERS =
			"[Environment Setting]"			       + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_OUTPUT_PATH        + "=%s"  + com.lhy.tool.Constants.LINE_SEPARATOR +
									 com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_TRAIN_CLUSER_FILE  + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_TRAIN_RATING_FILE  + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_TEST_FILE          + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
									com.lhy.tool.Constants.LINE_SEPARATOR +
			"[Latent Features]"			      + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_TEL_FEA_INIT_FILE  + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_PROG_FEA_INIT_FILE + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
									com.lhy.tool.Constants.LINE_SEPARATOR +
			"[PMF Model Setting]"			      + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_MODEL              + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_CONVERGE_THRESHOLD + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_FEATURE_NUM	      + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_GROUP_NUM	      + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_TELEVISION_NUM     + "=%s" + com.lhy.tool.Constants.LINE_SEPARATOR +
			TVConstants.STRING_PROGRAM_NUM	      + "=%s";

	private File outputPath       = null;
	private File trainClusterFile = null;
	private File trainRatingFile  = null;
	private File testFile         = null;
	private File telFeaInitFile   = null;
	private File progFeaInitFile  = null;
	private String modelName      = null;
	private int featureNum        = -1;
	private int groupNum          = -1;
	private int televisionNum     = -1;
	private int programNum        = -1;
	private Float convergeThreshold = null;

	public ParamManager() {}

	public Float getConvergeThreshold() {
		return convergeThreshold;
	}

	public int getFeatureNum() {
		return featureNum;
	}

	public int getGroupNum() {
		return groupNum;
	}

	public File getOutputPath() {
		return outputPath;
	}

	public File getProgramFeatureInitFile() {
		return progFeaInitFile;
	}

	public int getProgramNum() {
		return programNum;
	}

	public String getModelName() {
		return modelName;
	}
	
	public File getTelevisionFeatureInitFile() {
		return telFeaInitFile;
	}

	public int getTelevisionNum() {
		return televisionNum;
	}

	public File getTestFile() {
		return testFile;
	}

	public File getTrainClusterFile() {
		return trainClusterFile;
	}

	public File getTrainRatingFile() {
		return trainRatingFile;
	}

	public void setConvergeThreshold(float convergeThreshold) {
		this.convergeThreshold = convergeThreshold;
	}

	public void setFeatureNum(int featureNum) {
		this.featureNum = featureNum;
	}

	public void setGroupNum(int groupNum) {
		this.groupNum = groupNum;
	}

	public void setOutputPath(File outputPath) {
		this.outputPath = outputPath;
	}
	
	public void setModelName(String modelName) {
		this.modelName = modelName;
	}

	public void setProgramFeatureInitFile(File progFeaInitFile) {
		this.progFeaInitFile = progFeaInitFile;
	}

	public void setProgramNum(int programNum) {
		this.programNum = programNum;
	}

	public void setTelevisionNum(int televisionNum) throws TVException {
		this.televisionNum = televisionNum;
	}

	public void setTelevisionFeatureInitFile(File telFeaInitFile) {
		this.telFeaInitFile = telFeaInitFile;
	}

	public void setTestFile(File testFile) throws TVException{
		this.testFile = testFile;
		if (! Utils.exists(testFile)) {
			throw new TVException("Test file does not exist.[%s]",
				testFile == null ? null : testFile.getAbsolutePath());
		}
	}

	public void setTrainClusterFile(File trainClusterFile)
						throws TVException {
		this.trainClusterFile = trainClusterFile;
		if (! Utils.exists(trainClusterFile)) {
			throw new TVException("Train cluster file does not exist.[%s]",
				trainClusterFile == null ? null : trainClusterFile.getAbsolutePath());
		}
	}

	public void setTrainRatingFile(File trainFile) throws TVException {
		this.trainRatingFile = trainFile;
		if (! Utils.exists(trainFile)) {
			throw new TVException("Train rating file does not exist.[%s]",
				trainFile == null ? null : trainFile.getAbsolutePath());
		}
	}

	public void load(File inputFile) throws TVException {
		if (! Utils.exists(inputFile)) {
			throw new TVException(String.format("Config file doesn't exist. [ %s ]",
				inputFile == null ? inputFile : inputFile.getAbsolutePath()));
		}

		BufferedReader confReader = null;
		try {
			confReader       = Utils.createBufferedReader(inputFile);
			Properties props = new Properties();
			props.load(confReader);

			String sprop = props.getProperty(TVConstants.STRING_OUTPUT_PATH);
			if (! Utils.isEmpty(sprop)) {
				setOutputPath(new File(sprop));
			}

			sprop = props.getProperty(TVConstants.STRING_TRAIN_CLUSER_FILE);
			if (! Utils.isEmpty(sprop)) {
				setTrainClusterFile(new File(sprop));
			} else {
				Utils.err("No specified train cluster file.");
			}

			sprop = props.getProperty(TVConstants.STRING_TRAIN_RATING_FILE);
			if (! Utils.isEmpty(sprop)) {
				setTrainRatingFile(new File(sprop));
			} else {
				throw new TVException("No specified train rating file.");
			}

			sprop = props.getProperty(TVConstants.STRING_TEST_FILE);
			if (! Utils.isEmpty(sprop)) {
				setTestFile(new File(sprop));
			}

			sprop = props.getProperty(TVConstants.STRING_TEL_FEA_INIT_FILE);
			if (! Utils.isEmpty(sprop)) {
				setTelevisionFeatureInitFile(new File(sprop));
			}
	
			sprop = props.getProperty(TVConstants.STRING_PROG_FEA_INIT_FILE);
			if (! Utils.isEmpty(sprop)) {
				setProgramFeatureInitFile(new File(sprop));
			}
	
			sprop = props.getProperty(TVConstants.STRING_FEATURE_NUM);
			if (! Utils.isEmpty(sprop)) {
				setFeatureNum(Integer.parseInt(sprop));
			} else {
				Utils.err("No feature num specified, using default value instead.");
			}

			sprop = props.getProperty(TVConstants.STRING_GROUP_NUM);
			if (! Utils.isEmpty(sprop)) {
				setGroupNum(Integer.parseInt(sprop));
			} else {
				Utils.err("No group num specified.");
			}

			sprop = props.getProperty(TVConstants.STRING_TELEVISION_NUM);
			if (! Utils.isEmpty(sprop)) {
				setTelevisionNum(Integer.parseInt(sprop));
			} else {
				throw new TVException("No television num specified.");
			}
	
			sprop = props.getProperty(TVConstants.STRING_PROGRAM_NUM);
			if (! Utils.isEmpty(sprop)) {
				setProgramNum(Integer.parseInt(sprop));
			} else {
				throw new TVException("No program num specified.");
			}

			sprop = props.getProperty(TVConstants.STRING_CONVERGE_THRESHOLD);
			if (! Utils.isEmpty(sprop)) {
				setConvergeThreshold(Float.parseFloat(sprop)); 
			}
			
			setModelName(props.getProperty(TVConstants.STRING_MODEL));
		} catch (NumberFormatException e) {
			throw new TVException (e.toString());
		} catch (IOException e) {
			throw new TVException (e.toString());
		} finally {
			Utils.cleanup(confReader);
		}
	}

	public boolean store(File outputFile) {
		if (outputFile != null) {
			BufferedWriter writer = null;
			try {
				writer = Utils.createBufferedWriter(outputFile);

				Utils.write(writer, toString(), true);

				return true;
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				Utils.cleanup(writer);
			}
		}

		return false;
	}

	@Override
	public String toString() {
		return String.format(FORMAT_PARAMETERS,
				Utils.wrapPath(outputPath),
				Utils.wrapPath(trainClusterFile),
				Utils.wrapPath(trainRatingFile),
				Utils.wrapPath(testFile),
				Utils.wrapPath(telFeaInitFile),
				Utils.wrapPath(progFeaInitFile),
				modelName,
				convergeThreshold,
				featureNum,
				groupNum,
				televisionNum,
				programNum);
	}

	@Override
	public ParamManager clone() {
		ParamManager paramManager = new ParamManager();

		try {
			paramManager.setOutputPath(outputPath);
			paramManager.setTrainRatingFile(trainRatingFile);
			paramManager.setTestFile(testFile);
			paramManager.setTelevisionFeatureInitFile(telFeaInitFile);
			paramManager.setProgramFeatureInitFile(progFeaInitFile);
			paramManager.setModelName(modelName);
			paramManager.setFeatureNum(featureNum);
			paramManager.setGroupNum(groupNum);
			paramManager.setTelevisionNum(televisionNum);
			paramManager.setProgramNum(programNum);
			paramManager.setConvergeThreshold(convergeThreshold);
		} catch (TVException e) {
			// never reaches
			e.printStackTrace();
		}

		return paramManager;
	}
}

